package org.delta.winery.harvestwine.facade;

import org.delta.dto.entities.grape.AddGrapeDTO;
import org.delta.dto.entities.grape.GrapeDTO;
import org.delta.dto.entities.harvest.AddHarvestDTO;
import org.delta.dto.entities.harvest.HarvestListEntityDTO;
import org.delta.dto.entities.harvest.UpdateHarvestDTO;
import org.delta.winery.harvestwine.data.mapping.GrapeMapping;
import org.delta.winery.harvestwine.data.mapping.HarvestMapping;
import org.delta.winery.harvestwine.service.HarvestManagementService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Optional;

@Service
public class HarvestFacadeImpl implements HarvestFacade{

    final HarvestManagementService harvestService;

    public HarvestFacadeImpl(HarvestManagementService harvestService) {
        this.harvestService = harvestService;
    }

    public ResponseEntity<Collection<HarvestListEntityDTO>> getHarvestsWithParams(Optional<Long> grapeId, Optional<Integer> year) {
        return new ResponseEntity<>(harvestService.getHarvestsByGrapeAndYear(grapeId, year)
                .stream()
                .map(HarvestMapping::getHarvestListEntityDto).toList(), HttpStatus.OK);

    }

    public ResponseEntity<HarvestListEntityDTO> addHarvest(AddHarvestDTO harvest) {
        return new ResponseEntity<>(HarvestMapping.getHarvestListEntityDto(harvestService.addHarvest(harvest)), HttpStatus.CREATED);
    }


    public ResponseEntity<HarvestListEntityDTO> updateHarvest(UpdateHarvestDTO harvest) {
        return new ResponseEntity<>(HarvestMapping.getHarvestListEntityDto(harvestService.updateHarvest(harvest)), HttpStatus.OK);
    }

    @Transactional(readOnly = true)
    public ResponseEntity<HarvestListEntityDTO> getHarvest(Long id) {
        return new ResponseEntity<>(HarvestMapping.getHarvestListEntityDto(harvestService.getHarvest(id)), HttpStatus.OK);
    }

    public ResponseEntity<HarvestListEntityDTO> deleteHarvest(Long id) {
        return new ResponseEntity<>(HarvestMapping.getHarvestListEntityDto(harvestService.deleteHarvest(id)), HttpStatus.OK);
    }

    public ResponseEntity<Collection<GrapeDTO>> getGrapes() {
        return new ResponseEntity<>(harvestService.getGrapes().stream().map(GrapeMapping::getGrapeDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<GrapeDTO> addGrape(AddGrapeDTO grape) {
        return new ResponseEntity<>(GrapeMapping.getGrapeDto(harvestService.addGrape(grape)), HttpStatus.CREATED);
    }

    @Transactional(readOnly = true)
    public ResponseEntity<GrapeDTO> getGrape(String grapeName) {
        return new ResponseEntity<>(GrapeMapping.getGrapeDto(harvestService.getGrape(grapeName)), HttpStatus.OK);
    }

    public ResponseEntity<GrapeDTO> deleteGrape(String grapeName) {
        return new ResponseEntity<>(GrapeMapping.getGrapeDto(harvestService.deleteGrape(grapeName)), HttpStatus.OK);
    }

    public ResponseEntity<Boolean> canBeStored(BigDecimal volume, String bearer) {
        return new ResponseEntity<>(harvestService.canBeStored(volume, bearer), HttpStatus.OK);
    }

    public ResponseEntity<BigDecimal> surplusValue(BigDecimal volumeInLiters, BigDecimal purchasePricePerLiter, BigDecimal bottlePrice, Long id, String bearer) {
        return new ResponseEntity<>(harvestService.surplusValue(volumeInLiters, purchasePricePerLiter, bottlePrice, id, bearer), HttpStatus.OK);
    }

}
