from locust import SequentialTaskSet, task, TaskSet
from abc import abstractmethod
from requests import Response
from enum import Enum

class HttpRequest(Enum):
    POST = 0,
    PUT = 1,
    GET = 2,
    DELETE = 3

class BaseTaskSet(SequentialTaskSet):
    harvest_service = 'http://localhost:8081'
    production_service = 'http://localhost:8082'
    store_wine_service = 'http://localhost:8083'

    @abstractmethod
    def get_base_url(self):
        pass

    def on_start(self):
        f = open("./bearer.txt", "r")
        if not f:
            raise BaseException("File bearer.txt not found")
        self.bearer = f.read()
        if not self.bearer or self.bearer == '':
            raise BaseException("No bearer token found in bearer.txt")
    
    def http_request(self, endpoint, request=HttpRequest.GET, body={}) -> Response:
        base_url = self.get_base_url()
        name = base_url + endpoint.split('?')[0]
        if request == HttpRequest.GET:
            return self.client.get(base_url + endpoint,
                            headers={"Authorization": f"Bearer {self.bearer}"}, 
                            name=name)
        if request == HttpRequest.POST:
            return self.client.post(base_url + endpoint,
                            headers={"Authorization": f"Bearer {self.bearer}"}, 
                            json=body,
                            name=name)
        if request == HttpRequest.PUT:
            return self.client.put(base_url + endpoint,
                            headers={"Authorization": f"Bearer {self.bearer}"}, 
                            json=body,
                            name=name)
