from classes.BaseTaskSet import BaseTaskSet, HttpRequest
from locust import task
from locust.exception import ResponseError
import random

grape_names = [
    "Cabernet Sauvignon",
    "Chardonnay",
    "Merlot",
    "Pinot Noir",
    "Syrah",
    "Sauvignon Blanc",
    "Riesling",
    "Grenache",
    "Tempranillo",
    "Malbec",
    "Zinfandel",
    "Sangiovese",
    "Nebbiolo",
    "Petit Verdot",
    "Viognier",
    "Mourvèdre",
    "Cabernet Franc",
    "Gamay",
    "Chenin Blanc",
    "Semillon"
]

origins = [
    "Italy",
    "France",
    "Spain",
    "Portugal",
    "Germany",
    "Greece",
    "Austria",
    "Hungary",
    "Romania",
    "Switzerland",
    "Czech republic",
    "Slovakia"
]

class WineManagementTasks(BaseTaskSet):
    base_url = ''
    grape = {}
    container = {}
    
    def get_base_url(self):
        return self.base_url


    @task
    def get_grape(self):
        grape_name = random.choice(grape_names)
        self.base_url = self.harvest_service
        response = self.http_request('/grapes')
        if not response.ok:
            raise ResponseError
        
        for grape in response.json():
            if grape["grapeName"] == grape_name:
                self.grape = grape
                break
        
        if not bool(self.grape):
            response = self.http_request('/grapes', HttpRequest.POST, {"grapeName" : grape_name})
            if not response.ok:
                raise ResponseError
            
            self.grape = response.json()

    @task
    def add_container(self):
        self.base_url = self.harvest_service
        price = random.randrange(50,1000,10)
        year = random.randrange(1985, 2023)
        volume = random.randrange(100, 2000, 100)

        response = self.http_request('/harvest', 
                                    HttpRequest.POST,
                                    {
                                        "grape" : {
                                            "grapeName": self.grape["grapeName"],
                                            "grapeID" : int(self.grape["grapeID"])
                                        },
                                        "volume": volume,
                                        "price": price,
                                        "year" : year
                                    })
        
        self.base_url = self.harvest_service        
        response = self.http_request(f'/harvests/canStore?volume={volume}')
        
        self.base_url = self.production_service
        if response.ok and response.text.lower() != 'true':
            self.base_url = self.production_service
            response = self.http_request('/stored/containers', HttpRequest.POST, {"capacity" : volume, "isEmpty" : True})
            self.container = response.json()
        else:
            response = self.http_request(f'/stored/filter_containers?isEmpty=true')
            if not response.ok:
                raise ResponseError
            
            if len(response.json()) == 0:
                response = self.http_request('/stored/containers', HttpRequest.POST, {"capacity" : volume, "isEmpty" : True})
                self.container = response.json()
            else:
                self.container = response.json()[0]

    @task
    def add_and_update_wine(self):
        volume = random.randrange(50, 800, 10)
        year = 2023
        grape_type = self.grape["grapeName"]
        grape_name = f'{grape_type} {year} {volume}'
        wine_state = 'AGING_WINE'
        self.base_url = self.production_service
        response = self.http_request('/stored/wines?id=0&capacity=0&empty=true', 
                        HttpRequest.POST,
                        {
                            "name" : grape_name,
                            "volume": volume,
                            "year": year,
                            "grapeType": grape_type,
                            "wineState": wine_state
                        })
        
        if not response.ok:
            print(response.json())
            raise ResponseError
        
        wine = response.json()
        wine["wineState"] = 'AGED_WINE'
        response = self.http_request('/stored/wines', HttpRequest.PUT, wine)

        if not response.ok:
            print(response.json())
            raise ResponseError
    
    @task
    def create_bottle(self):
        self.base_url = self.store_wine_service
        country = random.choice(origins)
        year = 2023
        price = random.randrange(70, 500, 5)
        name = 'New wine'
        amount = random.randrange(10, 100)
        wine_type = random.choice(['RED', 'WHITE', 'ROSE'])
        response = self.http_request('/store/bottle', 
                        HttpRequest.POST,
                        {
                            "price": price,
                            "volume": 0.75,
                            "year": year,
                            "origin": country,
                            "amount": amount,
                            "name": name,
                            "wineType": wine_type
                        })
        
        if not response.ok:
            print(response.json())
            raise ResponseError