package org.delta.winery.sellwine;

import io.swagger.v3.oas.models.security.SecurityScheme;
import org.delta.dto.constants.Scopes;
import org.springdoc.core.customizers.OpenApiCustomizer;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configurers.oauth2.server.resource.OAuth2ResourceServerConfigurer;
import org.springframework.security.web.SecurityFilterChain;

@SpringBootApplication(scanBasePackages = "org.delta")
public class SellWine {
    public static void main(String[] args) {
        SpringApplication.run(SellWine.class);
    }

    @Bean
    SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http
                .authorizeHttpRequests(x -> x
                        .requestMatchers(HttpMethod.PUT).hasAuthority(Scopes.MANAGER)
                        .requestMatchers(HttpMethod.GET, "/sale/order", "/sale/bottle", "/sale/search", "/sale/bottle",
                                 "/sale/order/byId", "/sale/bottle/byId").hasAuthority(Scopes.MANAGER)
                        .requestMatchers(HttpMethod.POST, "/sale/order", "/sale/bottle", "/sale/bottle",
                                "/order/complete_order", "/order/leave_feedback").hasAuthority(Scopes.BUYER)
                        .requestMatchers(HttpMethod.DELETE, "/sale/order").hasAuthority(Scopes.MANAGER)
                        .anyRequest().permitAll()
                )
                .oauth2ResourceServer(OAuth2ResourceServerConfigurer::opaqueToken)
        ;
        return http.build();
    }
}
