package org.delta.winery.sellwine.data.model;

import jakarta.persistence.*;
import jakarta.validation.constraints.Size;
import org.delta.dto.enums.WineType;

import java.math.BigDecimal;
import java.util.HashSet;
import java.util.Set;

@Entity
@Table(name = "bottles")
public class Bottle {
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    @Column(name = "id")
    private Long id;

    @Column(name = "name",length = 100, nullable = false)
    private String name;

    @Column(name = "type", nullable = false)
    private WineType type;
    @Column(name = "price", nullable = false)
    private BigDecimal price;
    @Column(name = "volume", nullable = false)
    private BigDecimal volume;
    @Column(name = "bottled_year", nullable = false)
    private int year;
    @Size(min = 1)
    @Column(name = "origin", nullable = false)
    private String origin;
    @Column(name = "amount", nullable = false)
    private int amount;

    @ManyToMany
    @JoinTable(
            name = "Bottles_Orders",
            joinColumns = { @JoinColumn(name = "id") },
            inverseJoinColumns = { @JoinColumn(name = "order_id") }
    )
    private Set<Order> orders = new HashSet<>();

    public void setId(Long id) {
        this.id = id;
    }

    public Long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public WineType getType() {
        return type;
    }

    public void setType(WineType type) {
        this.type = type;
    }

    public BigDecimal getPrice() {
        return price;
    }

    public void setPrice(BigDecimal price) {
        this.price = price;
    }

    public BigDecimal getVolume() {
        return volume;
    }

    public void setVolume(BigDecimal volume) {
        this.volume = volume;
    }

    public int getYear() {
        return year;
    }

    public void setYear(int year) {
        this.year = year;
    }

    public String getOrigin() {
        return origin;
    }

    public void setOrigin(String origin) {
        this.origin = origin;
    }

    public int getAmount() {
        return amount;
    }

    public void setAmount(int amount) {
        this.amount = amount;
    }

    public void addOrder(Order order) {
        orders.add(order);
    }

    public void removeOrder(Order order) {
        orders.remove(order);
    }

    @Override
    public boolean equals(Object o) {
        if (o == this)
            return true;
        if (!(o instanceof Bottle bottle))
            return false;

        return bottle.getId() == this.id;
    }
}
