package org.delta.winery.sellwine.facade;

import org.delta.dto.entities.bottle.AddBottleDTO;
import org.delta.dto.entities.bottle.BottleDTO;
import org.delta.dto.entities.bottle.BottleListEntityDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.delta.dto.entities.order.*;
import org.delta.winery.sellwine.data.mapping.BottleMapping;
import org.delta.winery.sellwine.data.mapping.OrderMapping;
import org.delta.winery.sellwine.service.SaleManagementService;
import org.springframework.boot.actuate.autoconfigure.observation.ObservationProperties;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Optional;

@Service
public class SaleManagementFacadeImpl implements SaleManagementFacade {

    final SaleManagementService saleManagementService;

    public SaleManagementFacadeImpl(SaleManagementService saleManagementService) {
        this.saleManagementService = saleManagementService;
    }

    public ResponseEntity<Collection<BottleListEntityDTO>> getBottles() {
        return new ResponseEntity<>(saleManagementService.getBottles().stream().map(BottleMapping::getBottleListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<Collection<BottleListEntityDTO>> getBottles(Optional<String> origin, Optional<Integer> year) {
        return new ResponseEntity<>(saleManagementService.getBottles(origin, year)
                .stream()
                .map(BottleMapping::getBottleListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<Collection<OrderListEntityDTO>> getOrders() {
        return new ResponseEntity<>(saleManagementService.getOrders().stream().map(OrderMapping::getOrderListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<BottleDTO> addBottle(AddBottleDTO bottle) {
        return new ResponseEntity<>(BottleMapping.getBottleDto(saleManagementService.addBottle(bottle)), HttpStatus.CREATED);
    }

    public ResponseEntity<OrderDTO> addOrder(AddOrderDTO order) {
        return new ResponseEntity<>(OrderMapping.getOrderDto(saleManagementService.addOrder(order)), HttpStatus.CREATED);
    }

    public ResponseEntity<BottleDTO> updateBottle(UpdateBottleDTO bottle) {
        return new ResponseEntity<>(BottleMapping.getBottleDto(saleManagementService.updateBottle(bottle)), HttpStatus.OK);
    }

    public ResponseEntity<OrderDTO> addBottleToOrder(AddBottleToOrderDTO order) {
        return new ResponseEntity<>(OrderMapping.getOrderDto(saleManagementService.addBottleToOrder(order)), HttpStatus.OK);
    }

    public ResponseEntity<OrderDTO> removeBottleFromOrder(RemoveBottleFromOrderDTO order) {
        return new ResponseEntity<>(OrderMapping.getOrderDto(saleManagementService.removeBottleFromOrder(order)), HttpStatus.OK);
    }

    public ResponseEntity<OrderDTO> completeOrder(OrderBaseDTO order) {
        return new ResponseEntity<>(OrderMapping.getOrderDto(saleManagementService.completeOrder(order)), HttpStatus.OK);
    }
    public ResponseEntity<OrderDTO> leaveFeedbackOnOrder(UpdateOrderDTO order) {
        return new ResponseEntity<>(OrderMapping.getOrderDto(saleManagementService.leaveFeedbackOnOrder(order)), HttpStatus.OK);
    }

    @Transactional(readOnly = true)
    public ResponseEntity<BottleDTO> getBottle(Long id) {
        return new ResponseEntity<>(BottleMapping.getBottleDto(saleManagementService.getBottle(id)), HttpStatus.OK);
    }

    @Transactional(readOnly = true)
    public ResponseEntity<OrderDTO> getOrder(Long id) {
        return new ResponseEntity<>(OrderMapping.getOrderDto(saleManagementService.getOrder(id)), HttpStatus.OK);
    }

    public ResponseEntity<BottleDTO> deleteBottle(Long id) {
        return new ResponseEntity<>(BottleMapping.getBottleDto(saleManagementService.deleteBottle(id)), HttpStatus.OK);
    }

    public ResponseEntity<OrderDTO> deleteOrder(Long id) {
        return new ResponseEntity<>(OrderMapping.getOrderDto(saleManagementService.deleteOrder(id)), HttpStatus.OK);
    }
}
