package org.delta.Facade;

import org.delta.Data.Model.Bottle;
import org.delta.dto.entities.bottle.AddBottleDTO;
import org.delta.dto.entities.bottle.BottleDTO;
import org.delta.dto.entities.bottle.BottleListEntityDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.springframework.http.ResponseEntity;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;

public interface StoreFacade {

    /**
     * Get all stored bottles.
     *
     * @see Bottle
     */

    public ResponseEntity<List<BottleListEntityDTO>> getStoredBottles();

    /**
     * Get bottle with given id.
     *
     * @param id of bottle to get.
     * @return Bottle with given id.
     * @throws ResponseStatusException if bottle is not found
     * @see Bottle
     */
    public ResponseEntity<BottleDTO> getBottleById(long id);

    /**
     * Get bottle with given name.
     * @see Bottle
     * @param name of bottle to get.
     * @return Bottle with given name.
     * @throws ResponseStatusException if bottle is not found
     */
    public ResponseEntity<BottleDTO> getBottleByName(String name);

    /**
     * Add bottle to database.
     *
     * @param bottle parameter that will be stored into database.
     * @return Stored bottle entity with ID.
     * @throws ResponseStatusException if bottle cannot be created
     * @see Bottle
     */
    public ResponseEntity<BottleDTO> addBottle(AddBottleDTO bottle);

    /**
     * Update bottle in database.
     *
     * @param bottle Updated parameters of the bottle.
     * @throws ResponseStatusException if bottle to be updated was not found.
     *
     * @see UpdateBottleDTO
     */
    public ResponseEntity<BottleDTO> updateBottle(UpdateBottleDTO bottle);

    /**
     * Delete bottle with given id.
     * @param id of bottle to be deleted
     * @return Deleted bottle
     * @throws ResponseStatusException if bottle not found
     */
    public ResponseEntity<BottleDTO> deleteBottle(long id);
}
