package org.delta.Facade;

import org.delta.Data.Mapping.BottleMapping;
import org.delta.dto.entities.bottle.AddBottleDTO;
import org.delta.dto.entities.bottle.BottleDTO;
import org.delta.dto.entities.bottle.BottleListEntityDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.delta.service.StoreWineService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;


import java.util.List;

@Service
public class StoreFacadeImpl implements StoreFacade{
    private final StoreWineService storeWineService;

    public StoreFacadeImpl(StoreWineService storeWineService) {
        this.storeWineService = storeWineService;
    }


    public ResponseEntity<List<BottleListEntityDTO>> getStoredBottles() {
        return new ResponseEntity<>(storeWineService.getStoredBottles()
                .stream()
                .map(BottleMapping::getBottleListEntityDTO).toList(), HttpStatus.OK);
    }


    public ResponseEntity<BottleDTO> getBottleById(long id) {
        return new ResponseEntity<>(BottleMapping.getBottleDTO(storeWineService.getBottleById(id)), HttpStatus.OK);
    }


    public ResponseEntity<BottleDTO> getBottleByName(String name) {
        return new ResponseEntity<>(BottleMapping.getBottleDTO(storeWineService.getBottleByName(name)), HttpStatus.OK);
    }



    public ResponseEntity<BottleDTO> addBottle(AddBottleDTO bottle) {
        return new ResponseEntity<>(BottleMapping.getBottleDTO(storeWineService.addBottle(bottle)), HttpStatus.CREATED);
    }


    public ResponseEntity<BottleDTO> updateBottle(UpdateBottleDTO bottle) {
        return new ResponseEntity<>(BottleMapping.getBottleDTO(storeWineService.updateBottle(bottle)), HttpStatus.OK);

    }


    public ResponseEntity<BottleDTO> deleteBottle(long id) {
        return new ResponseEntity<>(BottleMapping.getBottleDTO(storeWineService.deleteBottle(id)), HttpStatus.OK);
    }



}
