package org.delta.service;

import org.delta.Data.Model.Bottle;
import org.delta.dto.entities.bottle.BottleDTO;
import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.delta.dto.entities.bottle.AddBottleDTO;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;

public interface StoreWineService {


    /**
     * Add bottle to database.
     *
     * @param bottle parameter that will be stored into database.
     * @return Stored bottle entity with ID.
     * @throws ResponseStatusException if bottle cannot be created
     * @see Bottle
     */
    Bottle addBottle(AddBottleDTO bottle);

    /**
     * Update bottle in database.
     *
     * @param bottle Updated parameters of the bottle.
     * @return Updated bottle
     * @throws ResponseStatusException if bottle to be updated was not found.
     * @see UpdateBottleDTO
     */
    Bottle updateBottle(UpdateBottleDTO bottle);

    /**
     * Get all stored bottles.
     *
     * @see Bottle
     */

    List<Bottle> getStoredBottles();

    /**
     * Get bottle with given id.
     *
     * @param id of bottle to get.
     * @return Bottle with given id.
     * @throws ResponseStatusException if bottle is not found
     * @see Bottle
     */

    Bottle getBottleById(long id);


    /**
     * Get bottle with given name.
     *
     * @param name of bottle to get.
     * @return Bottle with given name.
     * @throws ResponseStatusException if bottle is not found
     * @see Bottle
     */
    Bottle getBottleByName(String name);


    /**
     * Delete bottle with given id.
     *
     * @param id of bottle to be deleted
     * @return Deleted bottle
     * @throws ResponseStatusException if bottle not found
     */
    Bottle deleteBottle(long id);
}
