package org.delta.service;

import org.delta.Data.Mapping.BottleMapping;
import org.delta.Data.Model.Bottle;
import org.delta.Data.Repository.StoreRepository;
import org.delta.dto.entities.bottle.AddBottleDTO;

import org.delta.dto.entities.bottle.UpdateBottleDTO;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.math.BigDecimal;
import java.util.List;


@Service
public class StoreWineServiceImpl implements StoreWineService {

    private final StoreRepository storeRepository;

    public StoreWineServiceImpl(StoreRepository storeRepository) {
        this.storeRepository = storeRepository;
    }

    @Transactional
    public Bottle updateBottle(UpdateBottleDTO bottle) {
        Bottle updatedBottle = BottleMapping.getUpdateBottleDTO(bottle);
        if (updatedBottle.getVolume().compareTo(BigDecimal.valueOf(0)) <= 0) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Volume cannot be negative");
        }
        if (updatedBottle.getAmount() < 0) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Amount cannot be negative");
        }
        storeRepository.save(updatedBottle);
        return updatedBottle;

    }

    @Transactional
    public Bottle addBottle(AddBottleDTO bottle) {
        var bottleToAdd = BottleMapping.getAddBottleDTO(bottle);
        if (bottleToAdd.getVolume().compareTo(BigDecimal.valueOf(0)) <= 0) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Volume cannot be negative");
        }
        if (bottleToAdd.getAmount() < 0) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Amount cannot be negative");
        }
        return storeRepository.save(bottleToAdd);
    }

    @Transactional
    public List<Bottle> getStoredBottles() {
        return storeRepository.findAll();
    }

    @Transactional
    public Bottle getBottleById(long id) {
        return storeRepository.findById(id).orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Not valid id"));
    }

    @Transactional
    public Bottle getBottleByName(String name) {
        return storeRepository.findByName(name).orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Not valid name"));

    }

    @Transactional
    public Bottle deleteBottle(long id) {
        Bottle deletedBottle = storeRepository.findById(id).orElseThrow(() -> new ResponseStatusException(HttpStatus.BAD_REQUEST, "Not valid id"));
        storeRepository.deleteById(id);
        return deletedBottle;
    }
}
