package org.delta.winery.usermanagement.facade;

import org.delta.dto.entities.user.AddUserDTO;
import org.delta.dto.entities.user.UpdateUserDTO;
import org.delta.dto.entities.user.UserDetailDTO;
import org.delta.dto.entities.user.UserListEntityDTO;
import org.delta.dto.enums.UserRole;
import org.delta.winery.usermanagement.data.model.User;
import org.springframework.http.ResponseEntity;
import org.springframework.web.server.ResponseStatusException;

import java.util.Collection;
import java.util.Optional;

public interface UserFacade {
    /**
     * Get all users that are stored in database.
     * @return Collection of the users.
     */
    public ResponseEntity<Collection<UserListEntityDTO>> getUsers();

    /**
     * Get users based on optional filter parameters given.
     * @see User
     * @param username Filter users by username(if username filter is used,
     *                 only collection with maximum length 1 is always returned).
     * @param role Filter users by role(Manager or Buyer).
     * @param firstname Filter users by first name.
     * @param lastname Filter users by last name.
     * @return Collection of Users according to filter parameters given.
     */
    public ResponseEntity<Collection<UserListEntityDTO>> getUsers(Optional<String> username, Optional<UserRole> role, Optional<String> firstname, Optional<String> lastname);

    /**
     * Add user to database.
     * @see User
     * @param user User parameters that will be stored into database.
     * @return Stored user entity with ID.
     * @throws ResponseStatusException if user cannot be created
     */
    public ResponseEntity<UserDetailDTO> addUser(AddUserDTO user);


    /**
     * Get users in the role given.
     * @see User
     * @see UserRole
     * @param userRole user role.
     * @return collection of the users in user role given.
     */
    public ResponseEntity<Collection<UserListEntityDTO>> getUsersByRole(UserRole userRole);


    /**
     * Update user in database.
     * @see UpdateUserDTO
     * @param user Updated parameters of the user.
     * @throws ResponseStatusException if user to be updated was not found.
     */
    public ResponseEntity<UserDetailDTO> updateUser(UpdateUserDTO user);


    /**
     * Get specific user by username given.
     * @see User
     * @param username
     * @return User with username given in parameter.
     * @throws ResponseStatusException if user with username given does not exist.
     */
    public ResponseEntity<UserDetailDTO> getUser(String username);


    /**
     * Delete user by username.
     * @param userName
     * @throws ResponseStatusException if user with username given does not exist.
     */
    public ResponseEntity<UserDetailDTO> deleteUser(String userName);
}
