package org.delta.winery.usermanagement.facade;

import org.delta.dto.entities.user.AddUserDTO;
import org.delta.dto.entities.user.UpdateUserDTO;
import org.delta.dto.entities.user.UserDetailDTO;
import org.delta.dto.entities.user.UserListEntityDTO;
import org.delta.dto.enums.UserRole;
import org.delta.winery.usermanagement.data.mapping.UserMapping;
import org.delta.winery.usermanagement.service.UserManagementService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Optional;

@Service
public class UserFacadeIpl implements UserFacade{

    final UserManagementService userService;

    public UserFacadeIpl(UserManagementService userService) {
        this.userService = userService;
    }

    public ResponseEntity<Collection<UserListEntityDTO>> getUsers() {
        return new ResponseEntity<>(userService.getUsers().stream().map(UserMapping::getUserListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<Collection<UserListEntityDTO>> getUsers(Optional<String> username, Optional<UserRole> role, Optional<String> firstname, Optional<String> lastname) {
        return new ResponseEntity<>(userService.getUsers(username, role, firstname, lastname)
                .stream()
                .map(UserMapping::getUserListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<UserDetailDTO> addUser(AddUserDTO user) {
        return new ResponseEntity<>(UserMapping.getUserDetailDto(userService.addUser(user)), HttpStatus.CREATED);
    }

    public ResponseEntity<Collection<UserListEntityDTO>> getUsersByRole(UserRole userRole) {
        return new ResponseEntity<>(userService.getUsersByRole(userRole).stream().map(UserMapping::getUserListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<UserDetailDTO> updateUser(UpdateUserDTO user) {
        return new ResponseEntity<>(UserMapping.getUserDetailDto(userService.updateUser(user)),HttpStatus.OK);
    }

    @Transactional(readOnly = true)
    public ResponseEntity<UserDetailDTO> getUser(String username) {
        return new ResponseEntity<>(UserMapping.getUserDetailDto(userService.getUser(username)), HttpStatus.OK);
    }

    public ResponseEntity<UserDetailDTO> deleteUser(String userName) {
        return new ResponseEntity<>(UserMapping.getUserDetailDto(userService.deleteUser(userName)),HttpStatus.OK);
    }
}
