package org.delta.winery.usermanagement.test.service;

import org.delta.dto.entities.user.AddUserDTO;
import org.delta.dto.entities.user.UpdateUserDTO;
import org.delta.dto.enums.UserRole;
import org.delta.winery.usermanagement.data.mapping.UserMapping;
import org.delta.winery.usermanagement.data.model.User;
import org.delta.winery.usermanagement.data.repository.UserRepository;
import org.delta.winery.usermanagement.service.UserManagementService;
import org.delta.winery.usermanagement.service.UserManagementServiceImpl;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.Collection;
import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;


@SpringBootTest
public class UserManagementTests {


    @Autowired
    private UserManagementService userManagementService;

    private UserRepository userRepository;
    @BeforeEach
    public void setup(){
        userRepository = Mockito.mock(UserRepository.class);
        userManagementService= new UserManagementServiceImpl(userRepository);
    }

    @Test
    void getUsersTest(){
        when(userRepository.findAll()).thenReturn(List.of(getUser()));
        Collection<User> found = userManagementService.getUsers();
        User user = found.iterator().next();
        assertEquals(1, found.size());
        assertEquals(-1, user.getId());
        assertEquals("email",user.getEmail());
        assertEquals("username",user.getUserName());
        assertEquals("firstname",user.getFirstName());
        assertEquals(UserRole.MANAGER,user.getRole());
        assertEquals("lastname",user.getLastName());
        assertEquals("middlename",user.getMiddleName());
        assertEquals("password",user.getPassword());
    }

    @Test
    void getUsersTestWithParameters(){
        when(userRepository.filterUsers(any(),any(),any(),any())).thenReturn(List.of(getUser()));
        Collection<User> found = userManagementService.getUsers(any(),any(),any(),any());
        User user = found.iterator().next();
        assertEquals(1, found.size());
        assertEquals(-1, user.getId());
        assertEquals("email",user.getEmail());
        assertEquals("username",user.getUserName());
        assertEquals("firstname",user.getFirstName());
        assertEquals(UserRole.MANAGER,user.getRole());
        assertEquals("lastname",user.getLastName());
        assertEquals("middlename",user.getMiddleName());
        assertEquals("password",user.getPassword());

    }

    @Test
    void addUserTest(){
        AddUserDTO addUser= new AddUserDTO();
        addUser.setEmail(getUser().getEmail());
        addUser.setUserName(getUser().getUserName());
        addUser.setFirstName(getUser().getFirstName());
        addUser.setRole(getUser().getRole());
        addUser.setLastName(getUser().getLastName());
        addUser.setMiddleName(getUser().getMiddleName());
        addUser.setPassword(getUser().getPassword());
        when(userRepository.save(UserMapping.getUserEntity(addUser))).thenReturn(getUser());

        var user= userManagementService.addUser(addUser);

        assertEquals("email",user.getEmail());
        assertEquals("username",user.getUserName());
        assertEquals("firstname",user.getFirstName());
        assertEquals(UserRole.MANAGER,user.getRole());
        assertEquals("lastname",user.getLastName());
        assertEquals("middlename",user.getMiddleName());
        assertEquals("password",user.getPassword());

    }

    @Test
    void getUsersByRoleTest(){
        when(userRepository.findUsersByRole(UserRole.MANAGER)).thenReturn(List.of(getUser()));

        var found = userManagementService.getUsersByRole(UserRole.MANAGER);
        User user = found.iterator().next();
        assertEquals(1, found.size());
        assertEquals(-1, user.getId());
        assertEquals("email",user.getEmail());
        assertEquals("username",user.getUserName());
        assertEquals("firstname",user.getFirstName());
        assertEquals(UserRole.MANAGER,user.getRole());
        assertEquals("lastname",user.getLastName());
        assertEquals("middlename",user.getMiddleName());
        assertEquals("password",user.getPassword());
    }
    @Test
    void updateUserTest(){
        when(userRepository.findByUsername(getUser().getUserName())).thenReturn(Optional.of(getUser()));

        UpdateUserDTO updateUser= new UpdateUserDTO();
        updateUser.setEmail(getUser().getEmail());
        updateUser.setUserName(getUser().getUserName());
        updateUser.setFirstName(getUser().getFirstName());
        updateUser.setRole(getUser().getRole());
        updateUser.setLastName(getUser().getLastName());
        updateUser.setMiddleName(getUser().getMiddleName());
        updateUser.setPassword(getUser().getPassword()+"2");
        when(userRepository.save(getUser())).thenReturn(getUser());

        userManagementService.updateUser(updateUser);
        User user = userManagementService.getUser(updateUser.getUserName());

        assertEquals("email",user.getEmail());
        assertEquals("username",user.getUserName());
        assertEquals("firstname",user.getFirstName());
        assertEquals(UserRole.MANAGER,user.getRole());
        assertEquals("lastname",user.getLastName());
        assertEquals("middlename",user.getMiddleName());
        assertEquals("password2",user.getPassword());

    }
    @Test
    void getUserTest(){
        when(userRepository.findByUsername(getUser().getUserName())).thenReturn(Optional.of(getUser()));

        User user = userManagementService.getUser(getUser().getUserName());
        assertEquals(-1, user.getId());
        assertEquals("email",user.getEmail());
        assertEquals("username",user.getUserName());
        assertEquals("firstname",user.getFirstName());
        assertEquals(UserRole.MANAGER,user.getRole());
        assertEquals("lastname",user.getLastName());
        assertEquals("middlename",user.getMiddleName());
        assertEquals("password",user.getPassword());
    }

    private User getUser(){
        User user = new User();
        user.setId((long) -1);
        user.setEmail("email");
        user.setUserName("username");
        user.setFirstName("firstname");
        user.setRole(UserRole.MANAGER);
        user.setLastName("lastname");
        user.setMiddleName("middlename");
        user.setPassword("password");
        return user;
    }
}
