package org.delta.winery.wineproduction.facade;

import org.delta.dto.entities.container.AddContainerDTO;
import org.delta.dto.entities.container.ContainerDTO;
import org.delta.dto.entities.container.ContainerListEntityDTO;
import org.delta.dto.entities.container.UpdateContainerDTO;
import org.delta.dto.entities.wine.AddWineDTO;
import org.delta.dto.entities.wine.UpdateWineDTO;
import org.delta.dto.entities.wine.WineDTO;
import org.delta.dto.entities.wine.WineListEntityDTO;
import org.delta.dto.enums.WineState;
import org.delta.winery.wineproduction.data.mapping.ProductionMapping;
import org.delta.winery.wineproduction.data.model.WineContainer;
import org.delta.winery.wineproduction.service.WineProductionService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.math.BigDecimal;
import java.util.Collection;
import java.util.Optional;
import java.util.OptionalInt;

@Service
public class ProductionFacadeImpl implements ProductionFacade {

    final WineProductionService productionService;

    public ProductionFacadeImpl(WineProductionService productionService) {
        this.productionService = productionService;
    }

    public ResponseEntity<Collection<WineListEntityDTO>> getWines() {
        return new ResponseEntity<>(productionService.getWines().stream().map(ProductionMapping::getWineListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<Collection<WineListEntityDTO>> getWines(Optional<String> name, Optional<WineState> state, OptionalInt year, Optional<String> grapeType) {
        return new ResponseEntity<>(productionService.getWines(name, state, year, grapeType)
                .stream()
                .map(ProductionMapping::getWineListEntityDto).toList(), HttpStatus.OK);
    }


    public ResponseEntity<WineDTO> addWine(AddWineDTO wine, WineContainer container) {
        return new ResponseEntity<>(ProductionMapping.getWineDto(productionService.addWine(wine, container)), HttpStatus.CREATED);
    }

    public ResponseEntity<Collection<WineListEntityDTO>> getWineByState(WineState state) {
        return new ResponseEntity<>(productionService.getWineByState(state).stream().map(ProductionMapping::getWineListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<WineDTO> updateWine(UpdateWineDTO wine) {
        return new ResponseEntity<>(ProductionMapping.getWineDto(productionService.updateWine(wine)),HttpStatus.OK);
    }

    @Transactional(readOnly = true)
    public ResponseEntity<WineDTO> getWine(Long id) {
        return new ResponseEntity<>(ProductionMapping.getWineDto(productionService.getWine(id)), HttpStatus.OK);
    }


    public ResponseEntity<WineDTO> deleteWine(Long id) {
        return new ResponseEntity<>(ProductionMapping.getWineDto(productionService.deleteWine(id)),HttpStatus.OK);
    }

    public ResponseEntity<Collection<ContainerListEntityDTO>> getContainers() {
        return new ResponseEntity<>(productionService.getWineContainers().stream().map(ProductionMapping::getContainerListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<Collection<ContainerListEntityDTO>> getContainers(Optional<Boolean> isEmpty, Optional<BigDecimal> capacity) {
        return new ResponseEntity<>(productionService.getWineContainers(isEmpty, capacity)
                .stream().map(ProductionMapping::getContainerListEntityDto).toList(), HttpStatus.OK);
    }

    public ResponseEntity<ContainerDTO> addContainer(AddContainerDTO container) {
        return new ResponseEntity<>(ProductionMapping.getContainerDto(productionService.addContainer(container)), HttpStatus.CREATED);
    }

    public ResponseEntity<ContainerDTO> updateContainer(UpdateContainerDTO container) {

        return new ResponseEntity<>(ProductionMapping.getContainerDto(productionService.updateContainer(container)), HttpStatus.OK);
    }

    @Transactional(readOnly = true)
    public ResponseEntity<ContainerDTO> getContainer(Long id) {
        return new ResponseEntity<>(ProductionMapping.getContainerDto(productionService.getContainer(id)), HttpStatus.OK);
    }


    public ResponseEntity<ContainerDTO> deleteContainer(Long id) {
        return new ResponseEntity<>(ProductionMapping.getContainerDto(productionService.deleteContainer(id)), HttpStatus.OK);
    }
}

