package org.delta.winery.wineproduction.tests.Service;

import org.delta.dto.entities.container.AddContainerDTO;
import org.delta.dto.entities.wine.AddWineDTO;
import org.delta.winery.wineproduction.data.model.Wine;
import org.delta.winery.wineproduction.data.model.WineContainer;
import org.delta.winery.wineproduction.data.repository.ContainerRepository;
import org.delta.winery.wineproduction.data.repository.ProductionRepository;
import org.delta.winery.wineproduction.service.WineProductionService;
import org.delta.winery.wineproduction.service.WineProductionServiceImpl;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.boot.test.context.SpringBootTest;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static org.springframework.test.util.AssertionErrors.assertEquals;

@SpringBootTest
public class ProduceWineServiceTest {


    private WineProductionService wineProductionService;

    private ContainerRepository containerRepository;

    private ProductionRepository productionRepository;


    @BeforeEach
    public void setup(){
        containerRepository = Mockito.mock(ContainerRepository.class);

        productionRepository = Mockito.mock(ProductionRepository.class);

        wineProductionService = new WineProductionServiceImpl(productionRepository, containerRepository);
    }

    @Test
    public void getWinesTest(){
        when(productionRepository.findAll())
                .thenReturn(List.of(createWine()));
        var resultList = wineProductionService.getWines();
        var result = resultList.iterator().next();

        assertEquals("",1, resultList.size());
        assertEquals("", 1L, result.getId());
        assertEquals("", "Test", result.getName());
    }

    @Test
    public void getWinesWithParametersTest(){
        when(productionRepository.filterWine(any(), any(), any(), any()))
                .thenReturn(List.of(createWine()));
        var resultList = wineProductionService.getWines(any(), any(), any(), any());
        var result = resultList.iterator().next();

        assertEquals("",1, resultList.size());
        assertEquals("", 1L, result.getId());
        assertEquals("", "Test", result.getName());
    }

    @Test
    public void addWineTest(){
        AddWineDTO wine = new AddWineDTO();
        WineContainer wineContainer = new WineContainer();

        wine.setName("Test");
        wine.setWineState("AGING_WINE");
        wine.setVolume(BigDecimal.valueOf(50F));
        wine.setYear(2023);
        wine.setGrapeType("type");

        wineContainer.setEmpty(true);
        wineContainer.setCapacity(BigDecimal.valueOf(1000F));
        wineContainer.setId(Long.valueOf(1));

        List<WineContainer> containers = new ArrayList<>();
        containers.add(wineContainer);

        when(productionRepository.save(any()))
                .thenReturn(createWine());
        when(containerRepository.findAll())
                .thenReturn(containers);


        var result = wineProductionService.addWine(wine, createEmptyWineContainer());

        assertEquals("", 1L, result.getId());
        assertEquals("", "Test", result.getName());
    }

    @Test
    public void getWineByStateTest(){
        when(productionRepository.findWineByState(any()))
                .thenReturn(List.of(createWine()));

        var resultList = wineProductionService.getWineByState(any());

        var result = resultList.iterator().next();

        assertEquals("",1, resultList.size());
        assertEquals("", 1L, result.getId());
        assertEquals("", "Test", result.getName());

    }

    @Test
    public void getWineContainersWithParametersTest(){
        when(containerRepository.filterContainers(any(),any()))
                .thenReturn(List.of(createEmptyWineContainer()));

        var resultList = wineProductionService.getWineContainers(any(),any());

        var result = resultList.iterator().next();

        assertEquals("",1, resultList.size());
        assertEquals("",BigDecimal.valueOf(500F),result.getCapacity());
        assertEquals("", true, result.getEmpty());
    }

    @Test
    public void getWineContainersTest(){
        when(containerRepository.findAll())
                .thenReturn(List.of(createEmptyWineContainer()));

        var resultList = wineProductionService.getWineContainers();

        var result = resultList.iterator().next();

        assertEquals("",1, resultList.size());
        assertEquals("",1L,result.getId());
        assertEquals("",BigDecimal.valueOf(500F),result.getCapacity());
        assertEquals("", true, result.getEmpty());
    }

    @Test
    public void addWineContainersTest(){
        AddContainerDTO addedContainer = new AddContainerDTO();
        addedContainer.setCapacity(BigDecimal.valueOf(500F));
        addedContainer.setEmpty(true);

        when(containerRepository.save(any()))
                .thenReturn(createEmptyWineContainer());

        var result = wineProductionService.addContainer(addedContainer);


        assertEquals("",BigDecimal.valueOf(500F),result.getCapacity());
        assertEquals("",1L,result.getId());
        assertEquals("", true, result.getEmpty());
    }

    @Test
    public void getContainerTest(){
        when(containerRepository.findContainerById(1L))
                .thenReturn(Optional.of(createEmptyWineContainer()));

        var result = wineProductionService.getContainer(1L);

        assertEquals("",BigDecimal.valueOf(500F),result.getCapacity());
        assertEquals("",1L,result.getId());
        assertEquals("", true, result.getEmpty());
    }

    @Test
    public void getWineTest(){
        when(productionRepository.findById(1L))
                .thenReturn(Optional.of(createWine()));

        var result = wineProductionService.getWine(1L);

        assertEquals("", 1L, result.getId());
        assertEquals("", "Test", result.getName());
    }



    private Wine createWine(){
        Wine testWine = new Wine();
        testWine.setId(1L);
        testWine.setName("Test");
        return testWine;
    }

    private WineContainer createEmptyWineContainer(){
        WineContainer testWineContainer = new WineContainer();
        testWineContainer.setId(1L);
        testWineContainer.setCapacity(BigDecimal.valueOf(500F));
        testWineContainer.setEmpty(true);
        return testWineContainer;
    }




}
